//@see http://www.nicovideo.jp/watch/sm22173430
// Put TexStarLite.x and TexStarLite.png in same directory(folder).

#define ParticleSize     0.3

#define AreaSize       350.0

#define RotationSpeed    3.0
#define RotationNoize    1.0

#define NoizeLevel       2.0
#define NoizeSpeed       1.0

#define Flicker          0.5
#define FlickerSpeed     1.0

#define FadeLength     200.0

#define GroundY        -15.0

#define MAX     2147483647

const vec3 Velocity = vec3(3.6, -12.0, 0.0);

uniform float time;
uniform vec3  cameraPosition;

varying float alpha;
varying float rot;

vec3 getRandom(float index) {
	vec3 random;

	//non linear congruential method
	int i = int(index);
	random.x = ((i<<2)+1)*(i+1) % MAX;

	int x = int(random.x);
	random.y = ((x<<2)+1)*(x+1) % MAX;

	int y = int(random.y);
	random.z = ((y<<2)+1)*(y+1) % MAX;

	random /= float(MAX);
	return random;
}

void main(void){
	gl_Position = gl_Vertex;

	float index = gl_Vertex.z;
	index += gl_Vertex.x + 3.0 * gl_Vertex.y;
	index *= 10.0;

	gl_Position.z = 0.0;

	rot = time * RotationSpeed * (1.0 - (sin(index) * RotationNoize)) + index * 6.0;

	//random
	vec3 base_pos = getRandom(index);

	//fall,slope
	base_pos     = fract(base_pos   + (Velocity * time / AreaSize));

	//noize
	float stime = time * NoizeSpeed;
	base_pos.x += (sin(stime * 0.80 + index)       + cos(stime * 0.5 + index) * 0.5) * (NoizeLevel / AreaSize);
	base_pos.z += (sin(stime * 0.45 + index) * 0.6 + cos(stime * 0.9 + index) * 0.8) * (NoizeLevel / AreaSize);

	base_pos.xz = fract(base_pos.xz);

	//translate
	cameraPosition /= AreaSize;
	vec3 fraction = fract(cameraPosition);
		 if(fraction.x>0.5 && fraction.x-base_pos.x>0.5) base_pos.x+=1.0;
	else if(fraction.x<0.5 && base_pos.x-fraction.x>0.5) base_pos.x-=1.0;
		 if(fraction.y>0.5 && fraction.y-base_pos.y>0.5) base_pos.y+=1.0;
	else if(fraction.y<0.5 && base_pos.y-fraction.y>0.5) base_pos.y-=1.0;
		 if(fraction.z>0.5 && fraction.z-base_pos.z>0.5) base_pos.z+=1.0;
	else if(fraction.z<0.5 && base_pos.z-fraction.z>0.5) base_pos.z-=1.0;
	base_pos += floor(cameraPosition);

	//scale
	base_pos.xyz *= AreaSize;

	//ground fade
	alpha = clamp((base_pos.y-GroundY) * 0.2, 0.0, 1.0);

//	gl_Position.xyz += base_pos.xyz;
	gl_Position.xyz  = base_pos.xyz;

	gl_Position = gl_ModelViewProjectionMatrix * gl_Position;

	//flicker
	float fstime  = time * FlickerSpeed * NoizeSpeed;
	float flicker = (sin(fstime * 5.0 + index) * 0.5 + 0.5) +
					(cos(fstime * 2.3 + index) * 0.5 + 0.5) * 0.5;
	flicker = mix(1, flicker, Flicker);
	gl_PointSize  = gl_Point.size / gl_Position.w * ParticleSize * flicker;

	//far fade
	alpha *= 0.3 + 0.7 * (1.0 - clamp((gl_Position.z - 50.0) / FadeLength, 0.0, 1.0));

	//near fade
	alpha *= clamp((gl_Position.z - 10.0) * 0.05, 0.0, 1.0);
}
